use std::io::{stdin, Read, Write};
use std::os::unix::net::UnixStream;
use std::path::PathBuf;

use color_eyre::eyre::Context;
use color_eyre::Result;

use crate::Request;

pub fn handle(runtime_dir: PathBuf) -> Result<()> {
    let mut buf = String::new();
    stdin()
        .read_to_string(&mut buf)
        .wrap_err("Failed to read stdin")?;

    let req: Request = toml::from_str(&buf).wrap_err("Failed to parse request")?;

    tracing::trace!("Received request: {:?}", req);

    let path = runtime_dir.join(&req.session).with_extension("s");
    tracing::debug!(path = %path.display());

    let mut socket = UnixStream::connect(&path)
        .wrap_err_with(|| format!("Failed to connect to daemon socket {}", path.display()))?;

    socket
        .write_all(buf.as_bytes())
        .wrap_err("Failed to send request")?;

    tracing::info!("Sent request to {}", path.display());

    Ok(())
}
